/*
 * Copyright (c) 2007 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )

#include "usbconfig.h"
#include "cdc162.h"
#include "serflash.h"
#include "msc162.h"

#define DEV_VCC_ON()	{\
	bit_clear(PORTC,PC6);\
	bit_set(DDRC,PC6);\
}

#define LCD_LED_ON()	{\
	bit_clear(PORTB,PB7);\
	bit_set(DDRB,PB7);\
}

#ifdef EP_TX
void dump_serflash() {
	uint8_t i,j,d;
	serflash_close();
	serflash_open(SERFLASH_READ);
	serflash_seek(0L,SEEK_CUR);
	for(i=0; i<16; i++) {
		d = (serflash_offset >> 16) & 0xff;
		usbcdc_put_hex(d);
		d = (serflash_offset >> 8) & 0xff;
		usbcdc_put_hex(d);
		d = serflash_offset & 0xff;
		usbcdc_put_hex(d);
		usbcdc_putc(':');
		for (j=0; j<16; j++) {
			usbcdc_putc(' ');
			d = serflash_proc(0);
			usbcdc_put_hex(d);
		}
		usbcdc_put_pstr(PSTR("\r\n"));
	}
	serflash_close();
}

#endif

int main(void)
{
    uint8_t ch,d;

    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;

    bit_set(REGCR, REGDIS); // disable internal regulator 

    bit_set(DDRB, PB2);
    bit_set(PORTB, PB2); // LED ON

    init_spi();
    DEV_VCC_ON();
    usbcdc_init();

    sei();

    msc_init();
    for(;;) {
        msc_poll();
#ifdef EP_TX
        usbcdc_poll();
	if (usbcdc_can_getc(1)) {
	    ch = usbcdc_getc();
	    switch (ch) {
	    case 'x':
		msc_print_status();
		break;
	    case 'c':
	    	usbcdc_put_pstr(PSTR("clear pointer:\r\n"));
		serflash_offset = 0;
		break;
	    case 'r':
	    	usbcdc_put_pstr(PSTR("status:"));
		msc_lock(MSC_OWNER_PROG);
		spi_enable();
		spi_write(SERFLASH_RDSR);
		d = spi_read();
		spi_disable();
	    	usbcdc_put_hex(d);
	    	usbcdc_put_pstr(PSTR("\r\n"));
		msc_unlock();
		break;
	    case 's':
	    	usbcdc_put_pstr(PSTR("signature read:"));
		msc_lock(MSC_OWNER_PROG);
		serflash_init();

	    	usbcdc_put_hex(serflash_signature[0]);
	    	usbcdc_putc(' ');
	    	usbcdc_put_hex(serflash_signature[1]);
	    	usbcdc_putc(' ');
	    	usbcdc_put_hex(serflash_signature[2]);
	    	usbcdc_put_pstr(PSTR("\r\n"));
		msc_unlock();
		break;
	    case 'd':
		msc_lock(MSC_OWNER_PROG);
		dump_serflash();
		msc_unlock();
		break;
	    case 'e':
		msc_lock(MSC_OWNER_PROG);
		serflash_erase_4k(serflash_offset);
		msc_unlock();
		break;
	    case 'D':
		{
			uint16_t x;
			for (x=0; x<8192; x++) {
				msc_lock(MSC_OWNER_PROG);
				dump_serflash();
				msc_unlock();
			}
		}
		break;
	    case '\r':
	    	usbcdc_putc(ch);
	    	usbcdc_putc('\n');
		break;
	    default: 
	    	usbcdc_putc(ch);
	    }
	}
#endif
    }
    return 0;
}

void callback_set_baud(long baud) {
	//Baud = baud;
}

